//
//  ViewController.swift
//  DemoPay
//
//  Created by Flex on 11/21/16.
//  Copyright © 2016 Flex. All rights reserved.
//

import UIKit


struct DBItem {
    var name=""
    var description=""
    var category=""
    var price=0.0
    var weight=""
    var barcode=""
    var qty=0
    var imageFile=""
    var image:UIImage?=nil
}

class ViewController: TransactioniewController, UITableViewDelegate, UITableViewDataSource{

    public static var instance:ViewController! = nil

    @IBOutlet var tvItems: UITableView!
    @IBOutlet var lbTotal: UILabel!

    var db: [DBItem] = []

    let database: [String] = [
        "1230001,Pen,0.99,item_pen.png",
        "1230002,Memo,0.99,item_memo.png",
        "1230003,Notepad,1.99,item_notepad.png",
        "1230004,T-Shirt,1.99,item_tshirt.png",
        "4966006627909,Thermometer,2.99,item_thermometer.png",
        "4901480440330,Tray,0.99,item_tray.png",
        "1230005,Belt,1.49,item_belt.png",
        "1230006,String,1.49,item_string.png",
        ]

    var dbgroups=[[DBItem]]()
    var dbcategories = [String]()

    var total = 0.0

    func loadDatabase() {
        do {
        let filePath = Bundle.main.resourcePath!.stringByAppendingPathComponent("Database")
        let file = try NSString(contentsOfFile: filePath.stringByAppendingPathComponent("database.xml"), encoding: String.Encoding.ascii.rawValue)

        let xml = XML(data: file.data(using: String.Encoding.ascii.rawValue)!)

        for node in xml[0].children {
            var group: [DBItem] = []
            let gname = node.attributes["name"]!

            for inode in node.children {
                var item = DBItem()
                item.imageFile = inode["picture"]!.text
                item.image=UIImage(contentsOfFile: filePath.stringByAppendingPathComponent(item.imageFile))
                item.name = inode["name"]!.text
                item.description = inode["description"]!.text
                if inode["barcode"] != nil {
                    item.barcode = inode["barcode"]!.text
                }
                item.price = Double(inode["price"]!.text)!
                item.weight = inode["weight"]!.text
                item.category = gname

                group.append(item)
            }

            dbcategories.append(gname)
            dbgroups.append(group)
        }

        }catch {
        }
    }

    func updateDisplay() {
        total = 0.0

        for g in dbgroups {
            for item in g {
                let price = item.price * Double(item.qty)
                total+=price
            }
        }
        lbTotal.text=NSString.init(format: "$%.2f", total) as String
    }

    func addItemByBarcode(barcode: String) {
        for sec in 0..<dbgroups.count {
            for row in 0..<dbgroups[sec].count {
                if dbgroups[sec][row].barcode == barcode {
                    dbgroups[sec][row].qty = dbgroups[sec][row].qty + 1
                    tvItems.scrollToRow(at: IndexPath(row: row, section: sec), at: .top, animated: true)
                    tvItems.reloadData()
                    updateDisplay()
                    return
                }
            }
        }

        self.showMessage("Error", message: "Barcode not found in database!")
    }


    func  barcodeData(_ barcode: String!, type: Int32) {
        addItemByBarcode(barcode: barcode)
    }

    func connectionState(_ state: Int32) {

        Progress.hide()

        if state==CONN_STATES.CONNECTED.rawValue
        {
            do {
                try lib.uiShowInitScreen()
            } catch {
            }
        }else
        {
        }
        tvItems.reloadData()
    }

    func newTransaction() {
        for sec in 0..<dbgroups.count {
            for row in 0..<dbgroups[sec].count {
                dbgroups[sec][row].qty = 0
            }
        }
        tvItems.reloadData()
        updateDisplay()
    }

    override func emvTransactionComplete(approved: Bool, receipt: String, card: DTFinancialCardInfo?) {

        defer {
            newTransaction()
        }

        if approved {
            if hasFiscalPrinter() {
                _ = fiscalPrint(card: true)
            }
            if hasESCPOSPrinter() {
                _ = escposPrint(card: true)
            }

            self.showMessage("Transaction Approved", message: receipt)
        }else {
            self.showMessage("Transaction Denied", message: receipt)
        }
    }

    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return dbcategories[section]
    }

    func numberOfSections(in tableView: UITableView) -> Int {
        return dbcategories.count
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dbgroups[section].count
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell=tvItems.dequeueReusableCell(withIdentifier: "PayCell", for: indexPath) as! PayCell

        let item = dbgroups[indexPath.section][indexPath.row]
//        let components=database[items[indexPath.row]].components(separatedBy: ",")

//        let filePath = Bundle.main.resourcePath!.stringByAppendingPathComponent("Images/"+components[3])

        cell.ivImage.image=item.image!
        cell.lbName.text=item.name
        cell.lbPrice.text=String(format:"%.2f",item.price)
        cell.lbDescription.text=item.description
        cell.lbWeight.text=item.weight
        cell.layer.backgroundColor = UIColor.clear.cgColor
        cell.contentView.backgroundColor = UIColor.clear
        cell.backgroundColor = .clear

        cell.btDelete.tag=indexPath.section*1000+indexPath.row;

        if item.qty != 0 {
            cell.btDelete.isHidden = false
            cell.lbQty.text = String(format: "%d", item.qty)
            cell.lbQty.isHidden = false
        } else {
            cell.btDelete.isHidden = true
            cell.lbQty.isHidden = true
        }

        return cell
    }

    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        setItemQty(section: indexPath.section, row: indexPath.row, qty: dbgroups[indexPath.section][indexPath.row].qty + 1)
    }


    public func setItemQty(section: Int, row: Int, qty: Int) {
        NSLog("Setting %d/%d from %d to %d", section, row, dbgroups[section][row].qty, qty)

        dbgroups[section][row].qty = qty
        updateDisplay()
        tvItems.reloadData()
    }

    func getSelectedItems() -> [DBItem] {
        var selected = [DBItem]()

        for g in dbgroups {
            for item in g {
                if item.qty > 0 {
                    selected.append(item)
                }
            }
        }
        return selected
    }

//    func escposPrint(card: DTFinancialCardInfo?) -> Bool {
      func escposPrint(card: Bool) -> Bool {
        if lib.getSupportedFeature(FEATURES.FEAT_PRINTING, error: nil) != FEAT_UNSUPPORTED {
            Progress.setMessage("Printing receipt\nPlease wait...");

            do {
                try lib.prnPrintText("{+B}{=C}TRANSACTION COMPLETE\n")
                //prepare printout
                try lib.prnPrintText("{=F0}{+B}{=C}INVOICE\nDATECS LTD\n")
                try lib.prnPrintDelimiter("-".utf8CString[0])

                for item in getSelectedItems() {
                    try lib.prnPrintText(String(format:"%02dx %@ {=R}$%0.2f",item.qty,item.name,item.price * Double(item.qty)))
                }

                try lib.prnPrintDelimiter("-".utf8CString[0])
                try lib.prnPrintText(String.init(format: "{+DW}{+DH}{+B}TOTAL: {=R}$%.02f", total))
                try lib.prnPrintDelimiter("*".utf8CString[0])
                if card {
                    try lib.prnPrintText("Paid by: card")
                } else {
                    try lib.prnPrintText("Paid by: cash")
                }

//                if card != nil
//                {
//                    if card!.cardholderName.length() > 0 {
//                        try lib.prnPrintText("Name: " + card!.cardholderName)
//                    }
//                    try lib.prnPrintText("PAN: " + card!.accountNumber)
//                    try lib.prnPrintText(String.init(format: "Expires: %02d/%02d", card!.expirationMonth, card!.expirationYear))
//                }
                try lib.prnPrintText("\n\n\n")
                try lib.prnPrintText("{+DW}{+DH}{+B}{=C}THANK YOU")

                try lib.prnFeedPaper(0);
                try lib.prnWaitPrintJob(30.0)
            } catch  {

            }
        }
        return true
    }

    func fiscalPrint(card: Bool) -> Bool {
        defer {
            Progress.hide()
        }

        Progress.show(self)

        guard fiscal != nil
            else { return false }

            do {
                try fiscal?.customCommand(90, data: "")
                //operator
                try fiscal?.customCommand(102, data: "1\t1\t")
                //open fiscal

                try fiscal?.customCommand(48, data: "12\t\t")


                total = 0.0

                for item in getSelectedItems() {
                    try fiscal?.customCommand(49, data: String(format: "%@\t1\t%d\t%.2f\t0\t\t0\t", item.name, item.qty, item.price))

                    let price = item.price * Double(item.qty)
                    total+=price
                }

//                for item in getSelectedItems() {
//                    try fiscal?.customCommand(52, data: String(format: "O_%@\t1\t%d\t%.2f\t0\t\t0\t", item.name, item.qty, item.price))
//
//                    try fiscal?.customCommand(52, data: "O_" + components[1] + "\t0\t"+components[2]+"\t1\t")
//
//                    if (i & 1) > 0 {
//                        try fiscal?.customCommand(52, data: "O_" + components[1] + "\t1\t"+components[2]+"\t1\t")
//                    } else {
//                        let price = (components[2] as NSString).doubleValue
//                        total += price
//                    }
//                }


                if card {
                    try fiscal?.customCommand(53, data: "1\t" + String(format: "%.2f", total) + "\t")
                } else {
                    try fiscal?.customCommand(53, data: "0\t" + String(format: "%.2f", total) + "\t")
                }

                try fiscal?.customCommand(56, data: "")
                return true
            }catch {
            }
        return false
    }

    @IBAction func onFinishTransaction() {
        if total == 0.0 {
            self.showMessage("Error", message: "Please scan items first")
            return
        }
        if lib.connstate != CONN_STATES.CONNECTED.rawValue {
            self.showMessage("Error", message: "Device not connected")
            return
        }
        if !hasPinpad() && !hasPrinter() {
            self.showMessage("Error", message: "No devices attached")
            return
        }

        startTransaction(amount: Int(self.total*100))
    }


    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        lib.addDelegate(self)
    }

    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)

        lib.removeDelegate(self)
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        ViewController.instance = self
        // Do any additional setup after loading the view, typically from a nib.
        //force to update
        //        connectionState(lib.connstate)
        tvItems.delegate=self
        tvItems.dataSource=self

        tvItems.backgroundColor = .clear
        tvItems.tableFooterView = UIView()

        loadDatabase()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
}

