#import "UHFViewController.h"

@implementation UHFViewController

-(void)displayAlert:(NSString *)title message:(NSString *)message
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil, nil];
    [alert show];
}

#define RF_COMMAND(operation,c) {if(!c){[self displayAlert:@"Operatin failed!" message:[NSString stringWithFormat:@"%@ failed, error %@, code: %d",operation,error.localizedDescription,(int)error.code]]; return;} }

static NSString *hexToString(NSString * label, const void *data, size_t length)
{
    const char HEX[]="0123456789ABCDEF";
    char s[20000];
    for(int i=0;i<length;i++)
    {
        s[i*3]=HEX[((uint8_t *)data)[i]>>4];
        s[i*3+1]=HEX[((uint8_t *)data)[i]&0x0f];
        s[i*3+2]=' ';
    }
    s[length*3]=0;

    if(label)
        return [NSString stringWithFormat:@"%@(%d): %s",label,(int)length,s];
    else
        return [NSString stringWithCString:s encoding:NSASCIIStringEncoding];
}

-(IBAction)clear:(id)sender
{
    [logView setText:@""];
}

-(void)uhfTagDetected:(uint16_t)pc epc:(NSData *)epc
{
    logView.backgroundColor=[UIColor colorWithRed:0 green:1 blue:0 alpha:0.3];

    NSString *cardID=[NSString stringWithFormat:@"%@",epc];
    int nReads = [cards[cardID] intValue];
    nReads++;
    cards[cardID] = [NSNumber numberWithInt:nReads];

    NSMutableString *log=[[NSMutableString alloc] init];

    for (NSString *card in cards.allKeys) {
        [log appendFormat:@"%05d:%@\n",[cards[card] intValue], card];
    }

    [logView setText:log];
}

-(IBAction)onClearLog:(id)sender
{
    logView.text=@"";
    cards=[[NSMutableDictionary alloc] init];
}

NSString *countries[]=
{
    @"Korea",
    @"North America",
    @"United States",
    @"Europe",
    @"Japan",
    @"China 1",
    @"China 2",
    @"Brazil",
};

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)thePickerView {
    return 1;  // Or return whatever as you intend
}

- (NSInteger)pickerView:(UIPickerView *)thePickerView numberOfRowsInComponent:(NSInteger)component {
    return sizeof(countries)/sizeof(countries[0]);
}

- (NSString *)pickerView:(UIPickerView *)thePickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
    return countries[row];
}

- (void)pickerView:(UIPickerView *)thePickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
    selectedCountry = (UHF_CONTRY)row;
}

UHF_CONTRY selectedCountry = UHF_CONTRY_US;

-(IBAction)onUpdateSettingsAndStart:(id)sender
{
    NSError *error;

    [self onClearLog:sender];

    [dtdev uhfClose:nil];
    RF_COMMAND(@"UHF Enable",[dtdev uhfInit:selectedCountry error:&error]);

    //refer to documentation of the module
    uint8_t cmdModSpeed[]={0x33,0xFF,0x00,0xFA,0x01,0x01};
    uint8_t cmdModSensitivity[]={0x33,0xFF,0x00,0xFA,0x03,0x01};
    uint8_t cmdModMultiTag[]={0x33,0xFF,0x00,0xFA,0x02,0x01};

    switch (modulationSegmentedControl.selectedSegmentIndex) {
        case 0:
            RF_COMMAND(@"UHF Set Modulation",[dtdev uhfCommand:cmdModSpeed length:sizeof(cmdModSpeed) error:&error]);
            break;
        case 1:
            RF_COMMAND(@"UHF Set Modulation",[dtdev uhfCommand:cmdModSensitivity length:sizeof(cmdModSensitivity) error:&error]);
            break;
        case 2:
            RF_COMMAND(@"UHF Set Modulation",[dtdev uhfCommand:cmdModMultiTag length:sizeof(cmdModMultiTag) error:&error]);
            break;
    }

    uint8_t cmdSetTx[]={0x16,0x00,0x00};
    cmdSetTx[2]=txPowerSlider.value*10;
    RF_COMMAND(@"UHF Set Tx",[dtdev uhfCommand:cmdSetTx length:sizeof(cmdSetTx) error:&error]);


    RF_COMMAND(@"UHF Start Read",[dtdev uhfStartAutoReadForTagType:UHF_TAG_TYPE_C maxNumberOfTags:0 maxTimeForTagging:0 repeatCycleCount:0 error:&error]);
}

-(IBAction)onTxPowerChanges:(id)sender
{
    txPowerLabel.text=[NSString stringWithFormat:@"Tx Power (%ddb)",(int)txPowerSlider.value];
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    cards=[[NSMutableDictionary alloc] init];
    logView.backgroundColor=[UIColor colorWithRed:0 green:0 blue:1 alpha:0.3];
    logView.text=@"";
}

-(void)viewWillDisappear:(BOOL)animated
{
    [dtdev uhfClose:nil];
}

-(void)viewDidLoad
{
    countryPicker.showsSelectionIndicator=true;
    [countryPicker selectRow:(int)UHF_CONTRY_US inComponent:0 animated:false];
    modulationSegmentedControl.selectedSegmentIndex=2;

    dtdev=[DTDevices sharedDevice];
    [dtdev addDelegate:self];
    [super viewDidLoad];
}


@end
